package org.apfloat;

import java.math.BigInteger;
import java.math.BigDecimal;
import java.io.PushbackReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @version 1.0.1
 * @author Mikko Tommila
 */

public class ApfloatTest
    extends ApfloatTestCase
{
    public ApfloatTest(String methodName)
    {
        super(methodName);
    }

    public static void main(String[] args)
    { 
        junit.textui.TestRunner.run(suite());
    }

    public static TestSuite suite()
    {
        TestSuite suite = new TestSuite();

        suite.addTest(new ApfloatTest("testLongConstructor"));
        suite.addTest(new ApfloatTest("testFloatConstructor"));
        suite.addTest(new ApfloatTest("testDoubleConstructor"));
        suite.addTest(new ApfloatTest("testStringConstructor"));
        suite.addTest(new ApfloatTest("testStreamConstructor"));
        suite.addTest(new ApfloatTest("testBigIntegerConstructor"));
        suite.addTest(new ApfloatTest("testBigDecimalConstructor"));
        suite.addTest(new ApfloatTest("testComplexMethods"));
        suite.addTest(new ApfloatTest("testRadix"));
        suite.addTest(new ApfloatTest("testPrecision"));
        suite.addTest(new ApfloatTest("testScale"));
        suite.addTest(new ApfloatTest("testAdd"));
        suite.addTest(new ApfloatTest("testSubtract"));
        suite.addTest(new ApfloatTest("testMultiply"));
        suite.addTest(new ApfloatTest("testDivide"));
        suite.addTest(new ApfloatTest("testFloor"));
        suite.addTest(new ApfloatTest("testCeil"));
        suite.addTest(new ApfloatTest("testTruncate"));
        suite.addTest(new ApfloatTest("testNumberValues"));
        suite.addTest(new ApfloatTest("testEqualDigits"));
        suite.addTest(new ApfloatTest("testCompareTo"));
        suite.addTest(new ApfloatTest("testEquals"));
        suite.addTest(new ApfloatTest("testHashCode"));
        suite.addTest(new ApfloatTest("testToString"));
        suite.addTest(new ApfloatTest("testWriteTo"));
        suite.addTest(new ApfloatTest("testSerialization"));

        return suite;
    }

    public static void testLongConstructor()
    {
        Apfloat a = new Apfloat(5);
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", Apfloat.INFINITE, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        a = new Apfloat(6, Apfloat.DEFAULT);
        assertEquals("6 radix", 10, a.radix());
        assertEquals("6 precision", Apfloat.INFINITE, a.precision());
        assertEquals("6 String", "6", a.toString(true));
        a = new Apfloat(7, Apfloat.DEFAULT, 12);
        assertEquals("7 radix", 12, a.radix());
        assertEquals("7 precision", Apfloat.INFINITE, a.precision());
        assertEquals("7 String", "7", a.toString(true));
    }

    public static void testFloatConstructor()
    {
        Apfloat a = new Apfloat(5.0f);
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", 8, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        a = new Apfloat(6.0f, Apfloat.DEFAULT);
        assertEquals("6 radix", 10, a.radix());
        assertEquals("6 precision", 8, a.precision());
        assertEquals("6 String", "6", a.toString(true));
        a = new Apfloat(7.0f, Apfloat.DEFAULT, 12);
        assertEquals("7 radix", 12, a.radix());
        assertEquals("7 precision", 7, a.precision());
        assertEquals("7 String", "7", a.toString(true));
    }

    public static void testDoubleConstructor()
    {
        Apfloat a = new Apfloat(5.0);
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", 16, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        a = new Apfloat(6.0, Apfloat.DEFAULT);
        assertEquals("6 radix", 10, a.radix());
        assertEquals("6 precision", 16, a.precision());
        assertEquals("6 String", "6", a.toString(true));
        a = new Apfloat(7.0, Apfloat.DEFAULT, 12);
        assertEquals("7 radix", 12, a.radix());
        assertEquals("7 precision", 15, a.precision());
        assertEquals("7 String", "7", a.toString(true));
    }

    public static void testStringConstructor()
    {
        Apfloat a = new Apfloat("5");
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", 1, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        a = new Apfloat("99", Apfloat.DEFAULT);
        assertEquals("99 radix", 10, a.radix());
        assertEquals("99 precision", 2, a.precision());
        assertEquals("99 String", "99", a.toString(true));
        a = new Apfloat("aa", Apfloat.DEFAULT, 12);
        assertEquals("aa radix", 12, a.radix());
        assertEquals("aa precision", 2, a.precision());
        assertEquals("aa String", "aa", a.toString(true));
    }

    public static void testStreamConstructor()
        throws IOException
    {
        PushbackReader in = new PushbackReader(new StringReader("5"));
        Apfloat a = new Apfloat(in);
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", 1, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        in = new PushbackReader(new StringReader("99a"));
        a = new Apfloat(in, Apfloat.DEFAULT);
        assertEquals("99 radix", 10, a.radix());
        assertEquals("99 precision", 2, a.precision());
        assertEquals("99 String", "99", a.toString(true));
        in = new PushbackReader(new StringReader("aa"));
        a = new Apfloat(in, Apfloat.DEFAULT, 12);
        assertEquals("aa radix", 12, a.radix());
        assertEquals("aa precision", 2, a.precision());
        assertEquals("aa String", "aa", a.toString(true));
    }

    public static void testBigIntegerConstructor()
    {
        Apfloat a = new Apfloat(BigInteger.valueOf(5));
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", Apfloat.INFINITE, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        a = new Apfloat(BigInteger.valueOf(6), Apfloat.DEFAULT);
        assertEquals("6 radix", 10, a.radix());
        assertEquals("6 precision", Apfloat.INFINITE, a.precision());
        assertEquals("6 String", "6", a.toString(true));
        a = new Apfloat(BigInteger.valueOf(7), Apfloat.DEFAULT, 12);
        assertEquals("7 radix", 12, a.radix());
        assertEquals("7 precision", Apfloat.INFINITE, a.precision());
        assertEquals("7 String", "7", a.toString(true));
    }

    public static void testBigDecimalConstructor()
    {
        Apfloat a = new Apfloat(new BigDecimal("5"));
        assertEquals("5 radix", 10, a.radix());
        assertEquals("5 precision", 1, a.precision());
        assertEquals("5 String", "5", a.toString(true));
        a = new Apfloat(new BigDecimal("6"), Apfloat.DEFAULT);
        assertEquals("6 radix", 10, a.radix());
        assertEquals("6 precision", 1, a.precision());
        assertEquals("6 String", "6", a.toString(true));
    }

    public static void testComplexMethods()
    {
        Apfloat a = new Apfloat(5);
        assertEquals("real", a, a.real());
        assertEquals("imag", Apfloat.ZERO, a.imag());
    }

    public static void testRadix()
    {
        Apfloat a = new Apfloat(2, 500, 5);
        assertEquals("radix", 5, a.radix());
    }

    public static void testPrecision()
    {
        Apfloat a = new Apfloat(5, 5);
        assertEquals("5 precision", 5, a.precision());
        a = a.precision(4);
        assertEquals("4 precision", 4, a.precision());
        a = a.precision(6);
        assertEquals("6 precision", 6, a.precision());
        a = new Apfloat(0);
        assertEquals("0 precision", Apfloat.INFINITE, a.precision());
    }

    public static void testScale()
    {
        Apfloat a = new Apfloat(9);
        assertEquals("9 scale", 1, a.scale());
        a = new Apfloat(10);
        assertEquals("10 scale", 2, a.scale());
        a = new Apfloat(0);
        assertEquals("0 scale", -Apfloat.INFINITE, a.scale());
    }

    public static void testAdd()
    {
        Apfloat a = new Apfloat(4),
                b = new Apfloat(5);
        assertEquals("4 + 5", new Apfloat(9), a.add(b));
        assertEquals("4 + 0", new Apfloat(4), a.add(new Apfloat(0)));
        assertEquals("0 + 4", new Apfloat(4), new Apfloat(0).add(a));

        a = new Apfloat("1");
        b = new Apfloat("0.00001");
        assertEquals("1 + 0.00001", new Apfloat(1), a.add(b), b);
        assertEquals("0.00001 + 1", new Apfloat(1), b.add(a), b);
    }

    public static void testSubtract()
    {
        Apfloat a = new Apfloat(4),
                b = new Apfloat(5);
        assertEquals("4 - 5", new Apfloat(-1), a.subtract(b));
        assertEquals("4 - 0", new Apfloat(4), a.subtract(new Apfloat(0)));
        assertEquals("0 - 4", new Apfloat(-4), new Apfloat(0).subtract(a));

        a = new Apfloat("1");
        b = new Apfloat("0.00001");
        assertEquals("1 - 0.00001", new Apfloat(1), a.subtract(b), b);
        assertEquals("0.00001 - 1", new Apfloat(-1), b.subtract(a), b);
    }

    public static void testMultiply()
    {
        Apfloat a = new Apfloat(4, Apfloat.INFINITE, 12),
                b = new Apfloat(5, Apfloat.INFINITE, 12);
        assertEquals("4 * 5", new Apfloat(20, Apfloat.DEFAULT, 12), a.multiply(b));
        assertEquals("4 * 0", new Apfloat(0), a.multiply(new Apfloat(0)));
        assertEquals("0 * 4", new Apfloat(0), new Apfloat(0).multiply(a));
        assertEquals("4 * 1", new Apfloat(4, Apfloat.DEFAULT, 12), a.multiply(new Apfloat(1, Apfloat.DEFAULT, 12)));
        assertEquals("1 * 4", new Apfloat(4, Apfloat.DEFAULT, 12), new Apfloat(1, Apfloat.DEFAULT, 12).multiply(a));
        assertEquals("4 * ONE", new Apfloat(4, Apfloat.DEFAULT, 12), a.multiply(Apfloat.ONE));
        assertEquals("ONE * 4", new Apfloat(4, Apfloat.DEFAULT, 12), Apfloat.ONE.multiply(a));
    }

    public static void testDivide()
    {
        Apfloat a = new Apfloat(4, Apfloat.INFINITE, 12),
                b = new Apfloat(6, Apfloat.INFINITE, 12);
        assertEquals("4 / 6", new Apfloat("0.8", Apfloat.DEFAULT, 12), a.divide(b));
        assertEquals("0 / 4", new Apfloat(0), new Apfloat(0).divide(a));
        assertEquals("4 / 1", new Apfloat(4, Apfloat.INFINITE, 12), a.divide(new Apfloat(1, Apfloat.DEFAULT, 12)));
        assertEquals("4 / ONE", new Apfloat(4, Apfloat.INFINITE, 12), a.divide(Apfloat.ONE));
        try
        {
            a.divide(new Apfloat(0));
            fail("Division by zero allowed");
        }
        catch (ArithmeticException ae)
        {
            // OK: division by zero
        }
        assertEquals("long / long", new Apfloat(1), new Apfloat("101010101010101010101010101010101010101").divide(new Apfloat("101010101010101010101010101010101010101")), new Apfloat(2e-38));
    }

    public static void testFloor()
    {
        Apfloat a = new Apfloat("1.1");
        assertEquals("1.1 floor", new Apfloat(1), a.floor());
        assertEquals("1.1 floor reverse", a.floor(), new Apfloat(1));
        a = new Apfloat(1);
        assertEquals("1 floor", new Apfloat(1), a.floor());
        a = new Apfloat("1.99999999999999");
        assertEquals("1.99999999999999 floor", new Apfloat(1), a.floor());
        a = new Apfloat(0);
        assertEquals("0 floor", new Apfloat(0), a.floor());
        a = new Apfloat("-1.1");
        assertEquals("-1.1 floor", new Apfloat(-2), a.floor());
    }

    public static void testCeil()
    {
        Apfloat a = new Apfloat("1.1");
        assertEquals("1.1 ceil", new Apfloat(2), a.ceil());
        a = new Apfloat(1);
        assertEquals("1 ceil", new Apfloat(1), a.ceil());
        a = new Apfloat("1.99999999999999");
        assertEquals("1.99999999999999 ceil", new Apfloat(2), a.ceil());
        a = new Apfloat(0);
        assertEquals("0 ceil", new Apfloat(0), a.ceil());
        a = new Apfloat("-1.1");
        assertEquals("-1.1 ceil", new Apfloat(-1), a.ceil());
    }

    public static void testTruncate()
    {
        Apfloat a = new Apfloat("1.1");
        assertEquals("1.1 truncate", new Apfloat(1), a.truncate());
        a = new Apfloat(1);
        assertEquals("1 truncate", new Apfloat(1), a.truncate());
        a = new Apfloat("1.99999999999999");
        assertEquals("1.99999999999999 truncate", new Apfloat(1), a.truncate());
        a = new Apfloat(0);
        assertEquals("0 truncate", new Apfloat(0), a.truncate());
        a = new Apfloat("-1.1");
        assertEquals("-1.1 truncate", new Apfloat(-1), a.truncate());
    }

    public static void testNumberValues()
    {
        Apfloat a = new Apfloat(5);
        assertEquals("5 longValue", 5, a.longValue());
        assertEquals("5 intValue", 5, a.intValue());
        assertEquals("5 shortValue", 5, a.shortValue());
        assertEquals("5 byteValue", 5, a.byteValue());
        assertEquals("5 floatValue", 5.0f, a.floatValue(), 0.0f);
        assertEquals("5 doubleValue", 5.0, a.doubleValue(), 0.0);
        a = new Apfloat(1000000000000L);
        assertEquals("1000000000000 intValue", Integer.MAX_VALUE, a.intValue());
        assertEquals("1000000000000 shortValue", Short.MAX_VALUE, a.shortValue());
        assertEquals("1000000000000 byteValue", Byte.MAX_VALUE, a.byteValue());
        a = new Apfloat(-1000000000000L);
        assertEquals("-1000000000000 intValue", Integer.MIN_VALUE, a.intValue());
        assertEquals("-1000000000000 shortValue", Short.MIN_VALUE, a.shortValue());
        assertEquals("-1000000000000 byteValue", Byte.MIN_VALUE, a.byteValue());
        a = new Apfloat("1e100");
        assertEquals("1e100 floatValue", Float.POSITIVE_INFINITY, a.floatValue(), 0.0f);
        a = new Apfloat("-1e100");
        assertEquals("-1e100 floatValue", Float.NEGATIVE_INFINITY, a.floatValue(), 0.0f);
        a = new Apfloat("1e1000");
        assertEquals("1e1000 doubleValue", Double.POSITIVE_INFINITY, a.doubleValue(), 0.0);
        a = new Apfloat("-1e1000");
        assertEquals("-1e1000 doubleValue", Double.NEGATIVE_INFINITY, a.doubleValue(), 0.0);
    }

    public static void testEqualDigits()
    {
        Apfloat a = new Apfloat(5),
                b = new Apfloat(6);
        assertEquals("5 eq 6", 0, a.equalDigits(b));
        a = new Apfloat(10);
        b = new Apfloat(11);
        assertEquals("10 eq 11", 1, a.equalDigits(b));
        a = new Apfloat(7);
        b = new Apfloat(7);
        assertEquals("7 eq 7", Apfloat.INFINITE, a.equalDigits(b));
    }

    public static void testCompareTo()
    {
        Apfloat a = new Apfloat(0),
                b = new Apfloat(1);
        assertEquals("0 cmp 1", -1, a.compareTo(b));
        a = new Apfloat(6);
        b = new Apfloat(5);
        assertEquals("6 cmp 5", 1, a.compareTo(b));
        a = new Apfloat(6);
        b = new Apfloat(6);
        assertEquals("6 cmp 6", 0, a.compareTo(b));
        a = new Apfloat(6);
        b = a.floor();
        assertEquals("6 cmp 6.floor()", 0, a.compareTo(b));

        Object o = a;
        assertEquals("6 cmp object 6", 0, a.compareTo(o));

        try
        {
            a.compareTo("bogus");
            fail("Comparison to different class allowed");
        }
        catch (ClassCastException cce)
        {
            // OK: class can't be cast to Apfloat
        }
    }

    public static void testEquals()
    {
        Apfloat a = new Apfloat(0),
                b = new Apfloat(1);
        assertEquals("0 == 1", false, a.equals(b));
        a = new Apfloat(6);
        b = new Apfloat(5);
        assertEquals("6 == 5", false, a.equals(b));
        a = new Apfloat(6);
        b = new Apfloat(6);
        assertEquals("6 == 6", true, a.equals(b));

        assertEquals("6 == something else", false, a.equals("bogus"));

        a = new Apfloat(0, 1, 10);
        b = new Apfloat(0, 1, 12);
        assertEquals("0 == 0 different radixes", true, a.equals(b));
        a = new Apfloat(1, 1, 10);
        b = new Apfloat(1, 1, 12);
        assertEquals("1 == 1 different radixes", true, a.equals(b));

        ApfloatContext ctx = (ApfloatContext) ApfloatContext.getGlobalContext().clone();
        a = new Apfloat(0);
        b = new Apfloat(1);
        ctx.setBuilderFactory(new org.apfloat.internal.LongBuilderFactory());
        ApfloatContext.setThreadContext(ctx);
        assertEquals("0 == 0 different implementations", true, a.equals(new Apfloat(0)));
        assertEquals("1 == 1 different implementations", true, b.equals(new Apfloat(1)));
        ApfloatContext.removeThreadContext();
    }

    public static void testHashCode()
    {
        Apfloat a = new Apfloat(0),
                b = new Apfloat(1);
        assertTrue("0 != 1", a.hashCode() != b.hashCode());
        a = new Apfloat(6);
        b = new Apfloat(5);
        assertTrue("5 != 6", a.hashCode() != b.hashCode());
        a = new Apfloat(6);
        b = new Apfloat(6);
        assertEquals("6 == 6", a.hashCode(), b.hashCode());
    }

    public static void testToString()
    {
        Apfloat a = new Apfloat(0);
        assertEquals("0", "0", "" + a);
        a = new Apfloat(6);
        assertEquals("6", "6", "" + a);
        a = new Apfloat(123456789, 9);
        assertEquals("123456789", "1.23456789e8", "" + a);
        a = new Apfloat(123456789, 9);
        assertEquals("123456789 pretty", "123456789", a.toString(true));
    }

    public static void testWriteTo()
        throws IOException
    {
        StringWriter out = new StringWriter();
        Apfloat a = new Apfloat(0);
        a.writeTo(out);
        a = new Apfloat(6);
        a.writeTo(out);
        a = new Apfloat(123456789, 9);
        a.writeTo(out);
        a.writeTo(out, true);
        assertEquals("string", "061.23456789e8123456789", out.toString());
    }

    private static String getString(char character, int length)
    {
        StringBuffer buffer = new StringBuffer(length);

        for (int i = 0; i < length; i++)
        {
            buffer.append(character);
        }

        return buffer.toString();
    }

    public static void testSerialization()
        throws IOException, ClassNotFoundException
    {
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        ObjectOutputStream out = new ObjectOutputStream(buffer);
        Apfloat a = new Apfloat(5);
        out.writeObject(a);
        out.close();
        ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(buffer.toByteArray()));
        Apfloat b = (Apfloat) in.readObject();
        assertEquals("5 equals", a, b);
        assertTrue("5 !=", a != b);

        a = new Apfloat(getString('a', 1000000) + ".a", Apfloat.DEFAULT, 12);
        Apfloat a2 = a.floor();
        buffer.reset();
        out = new ObjectOutputStream(buffer);
        out.writeObject(a);
        assertTrue("Data has been used: " + buffer.size() + " > 400000", buffer.size() > 400000);
        out.writeObject(a2);
        out.close();
        in = new ObjectInputStream(new ByteArrayInputStream(buffer.toByteArray()));
        b = (Apfloat) in.readObject();
        Apfloat b2 = (Apfloat) in.readObject();
        assertEquals("1000000 equals", a, b);
        assertTrue("1000000 !=", a != b);
        assertEquals("1000000 floor equals", a2, b2);
        assertTrue("1000000 floor !=", a2 != b2);
        assertTrue("Data has been shared: " + buffer.size() + " < 600000", buffer.size() < 600000);
    }
}
