package org.apfloat.internal;

import org.apfloat.*;
import org.apfloat.spi.*;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @version 1.0.1
 * @author Mikko Tommila
 */

public class Int3NTTConvolutionStrategyTest
    extends IntConvolutionStrategyTestCase
    implements IntRadixConstants
{
    public Int3NTTConvolutionStrategyTest(String methodName)
    {
        super(methodName);
    }

    public static void main(String[] args)
    { 
        junit.textui.TestRunner.run(suite());
    }

    public static TestSuite suite()
    {
        TestSuite suite = new TestSuite();

        suite.addTest(new Int3NTTConvolutionStrategyTest("testFull"));
        suite.addTest(new Int3NTTConvolutionStrategyTest("testTruncated"));
        suite.addTest(new Int3NTTConvolutionStrategyTest("testAuto"));
        suite.addTest(new Int3NTTConvolutionStrategyTest("testFullBig"));
        suite.addTest(new Int3NTTConvolutionStrategyTest("testTruncatedBig"));
        suite.addTest(new Int3NTTConvolutionStrategyTest("testAutoBig"));

        return suite;
    }

    public static void testFull()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            int b1 = BASE[radix] - (int) 1;
            DataStorage src1 = createDataStorage(new int[] { (int) 1, (int) 2, (int) 3, (int) 4 }),
                        src2 = createDataStorage(new int[] { (int) 5, (int) 6 }),
                        src9 = createDataStorage(new int[] { b1, b1, b1 }),
                        src99 = createDataStorage(new int[] { b1, b1, b1, b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Int3NTTConvolutionStrategy(radix, new IntTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src1, src2, 6);

            check("normal", radix, new int[] { 0, (int) 5, (int) 16, (int) 27, (int) 38, (int) 24 }, result);

            result = convolutionStrategy.convolute(src9, src99, 9);

            check("max", radix, new int[] { b1, b1, b1 - (int) 1, b1, b1, b1, 0, 0, (int) 1 }, result);
        }
    }

    public static void testTruncated()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            int b1 = BASE[radix] - (int) 1;
            DataStorage src1 = createDataStorage(new int[] { (int) 1, (int) 2, (int) 3, (int) 4 }),
                        src2 = createDataStorage(new int[] { (int) 5, (int) 6 }),
                        src9 = createDataStorage(new int[] { b1, b1, b1 }),
                        src99 = createDataStorage(new int[] { b1, b1, b1, b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Int3NTTConvolutionStrategy(radix, new IntTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src1, src2, 3);

            check("normal", radix, new int[] { 0, (int) 5, (int) 16 }, result);

            result = convolutionStrategy.convolute(src9, src99, 6);

            check("max", radix, new int[] { b1, b1, b1 - (int) 1, b1, b1, b1 }, result);
        }
    }

    public static void testAuto()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            int b1 = BASE[radix] - (int) 1;
            DataStorage src9 = createDataStorage(new int[] { b1, b1, b1 });

            ConvolutionStrategy convolutionStrategy = new Int3NTTConvolutionStrategy(radix, new IntTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src9, 6);

            check("max", radix, new int[] { b1, b1, b1 - (int) 1, 0, 0, (int) 1 }, result);
        }
    }

    public static void testFullBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            int b1 = BASE[radix] - (int) 1;
            int size1 = 500,
                size2 = 300;
            int[] array1 = new int[size1],
                      array2 = new int[size2],
                      array3 = new int[size1 + size2];
            for (int i = 0; i < size1; i++)
            {
                array1[i] = b1;
                array3[i] = b1 - (int) (i == size2 - 1 ? 1 : 0);
            }
            for (int i = 0; i < size2; i++)
            {
                array2[i] = b1;
                array3[i + size1] = (int) (i == size2 - 1 ? 1 : 0);
            }
            DataStorage src9 = createDataStorage(array1),
                        src99 = createDataStorage(array2);

            ConvolutionStrategy convolutionStrategy = new Int3NTTConvolutionStrategy(radix, new IntTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src99, size1 + size2);

            check("max", radix, array3, result);
        }
    }

    public static void testTruncatedBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            int b1 = BASE[radix] - (int) 1;
            int size1 = 500,
                size2 = 300;
            int[] array1 = new int[size1],
                      array2 = new int[size2],
                      array3 = new int[size1];
            for (int i = 0; i < size1; i++)
            {
                array1[i] = b1;
                array3[i] = b1 - (int) (i == size1 - 1 || i == size2 - 1 ? 1 : 0);
            }
            for (int i = 0; i < size2; i++)
            {
                array2[i] = b1;
            }
            DataStorage src9 = createDataStorage(array1),
                        src99 = createDataStorage(array2);

            ConvolutionStrategy convolutionStrategy = new Int3NTTConvolutionStrategy(radix, new IntTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src99, src9, size1);

            check("max", radix, array3, result);
        }
    }

    public static void testAutoBig()
    {
        for (int radix = Character.MIN_RADIX; radix <= Character.MAX_RADIX; radix++)
        {
            int b1 = BASE[radix] - (int) 1;
            int size = 500;
            int[] array1 = new int[size],
                      array2 = new int[2 * size];
            for (int i = 0; i < size; i++)
            {
                array1[i] = b1;
                array2[i] = b1 - (int) (i == size - 1 ? 1 : 0);
                array2[i + size] = (int) (i == size - 1 ? 1 : 0);
            }
            DataStorage src9 = createDataStorage(array1);

            ConvolutionStrategy convolutionStrategy = new Int3NTTConvolutionStrategy(radix, new IntTableFNTStrategy());

            DataStorage result = convolutionStrategy.convolute(src9, src9, 2 * size);

            check("max", radix, array2, result);
        }
    }
}
